#ifndef _LLMPIDT
#define _LLMPIDT

#define MPIR_HAS_COOKIES
#ifdef MPIR_HAS_COOKIES
#define MPIR_COOKIE unsigned long cookie;
#define MPIR_SET_COOKIE(obj,value) (obj)->cookie = (value);
#else
#define MPIR_COOKIE
#define MPIR_SET_COOKIE(obj,value)
#endif


typedef enum {
  MPIR_NO = 0, MPIR_YES
} MPIR_BOOL;

/* 
   MPIR_FORT_INT is a special integer type for systems where sizeof(int) !=
   sizeof(Fortran REAL).  See the Fortran standard for why this is required.

   MPIR_LOGICAL is required for Fortran, since while the length of a 
   Fortran LOGICAL == Fortran INTEGER, the true/false values are 
   unspecified 
 */
typedef enum {
    MPIR_INT, MPIR_FLOAT, MPIR_DOUBLE, MPIR_COMPLEX, MPIR_LONG, MPIR_SHORT,
    MPIR_CHAR, MPIR_BYTE, MPIR_UCHAR, MPIR_USHORT, MPIR_ULONG, MPIR_UINT,
    MPIR_CONTIG, MPIR_VECTOR, MPIR_HVECTOR, MPIR_INDEXED,
    MPIR_HINDEXED, MPIR_STRUCT, MPIR_DOUBLE_COMPLEX, MPIR_PACKED, 
	MPIR_UB, MPIR_LB, MPIR_LONGDOUBLE, MPIR_LONGLONGINT, 
    MPIR_LOGICAL, MPIR_FORT_INT 
} MPIR_NODETYPE;


/* Must be able to hold any valid address.  64 bit machines may need
   to change this */
typedef long MPI_Aint;


struct MPIR_DATATYPE {
    MPIR_NODETYPE dte_type; /* type of datatype element this is */
    MPIR_COOKIE             /* Cookie to help detect valid item */
    MPIR_BOOL    committed; /* whether committed or not */
    int          is_contig; /* whether entirely contiguous */
    int              basic; /* Is this a basic type */
    int          permanent; /* Is this a permanent type */
    MPI_Aint        ub, lb; /* upper/lower bound of type */
    MPI_Aint real_ub, real_lb; /* values WITHOUT TYPE_UB/TYPE_LB */
    int             has_ub; /* Indicates that the datatype has a TYPE_UB */
    int             has_lb; /* Indicates that the datatype has a TYPE_LB */
    int             extent; /* extent of this datatype */
    int               size; /* size of type */
    int           elements; /* number of basic elements */
    int          ref_count; /* nodes depending on this node */
    int              align; /* alignment needed for start of datatype */
    int              count; /* replication count */
    MPI_Aint        stride; /* stride, for VECTOR and HVECTOR types */
    MPI_Aint      *indices; /* array of indices, for (H)INDEXED, STRUCT */
    int           blocklen; /* blocklen, for VECTOR and HVECTOR types */
    int         *blocklens; /* array of blocklens for (H)INDEXED, STRUCT */
    MPI_Datatype old_type;  /* type this type is built of, if 1 */
    MPI_Datatype *old_types;/* array of types, for STRUCT */
};

#endif
